import { HttpError } from './errors.mjs';
import { asyncGeneratorOf } from './util/async-generator-of.mjs';
import { calculateByteLength } from './util/calculate-byte-length.mjs';
import { asyncIterableMap } from './util/async-iterable-map.mjs';
import { asyncIterableChain } from './util/async-iterable-chain.mjs';
import { DEFAULT_TRANSFORM_RESULT_FN } from './transform-result.mjs';
import { asyncIterableFinalValueFromError } from './util/async-iterable-final-value-from-error.mjs';
import { GraphQLError } from "graphql";
export function toResponseResponsePayload(result, transformResult = DEFAULT_TRANSFORM_RESULT_FN) {
    const data = JSON.stringify(transformResult(result.payload));
    return {
        headers: {
            ...Object.fromEntries(result.headers.map(({ name, value }) => [name, value])),
            "content-type": "application/json",
            "content-length": String(calculateByteLength(data)),
        },
        source: asyncGeneratorOf(data),
        status: result.status,
    };
}
export function toMultiPartResponsePayload(result, transformResult = DEFAULT_TRANSFORM_RESULT_FN) {
    return {
        headers: {
            // prettier-ignore
            "Connection": "keep-alive",
            "Content-Type": 'multipart/mixed; boundary="-"',
            "Transfer-Encoding": "chunked",
        },
        source: asyncIterableChain(asyncGeneratorOf("---"), asyncIterableMap(result, (value) => {
            const chunk = JSON.stringify(transformResult(value));
            const data = [
                "",
                "Content-Type: application/json; charset=utf-8",
                "Content-Length: " + calculateByteLength(chunk),
                "",
                chunk,
            ];
            if (value.hasNext) {
                data.push("---");
            }
            return data.join("\r\n");
        }), asyncGeneratorOf("\r\n-----\r\n")),
        status: 200,
    };
}
export function toPushResponsePayload(result, transformResult = DEFAULT_TRANSFORM_RESULT_FN) {
    return {
        headers: {
            "Content-Type": "text/event-stream",
            // prettier-ignore
            "Connection": "keep-alive",
            "Cache-Control": "no-cache",
        },
        source: asyncIterableFinalValueFromError(asyncIterableMap(result, (value) => `data: ${JSON.stringify(transformResult(value))}\n\n`), (err) => {
            var _a, _b;
            return `data: ` +
                JSON.stringify(transformResult({
                    errors: [new GraphQLError((_b = (_a = err) === null || _a === void 0 ? void 0 : _a.message) !== null && _b !== void 0 ? _b : String(err))],
                })) +
                `\n\n`;
        }),
        status: 200,
    };
}
/**
 * Transforms the a `ProcessRequestResult` into a text stream published via an AsyncGenerator that can be piped into any HTTP framework.
 */
export function toResponsePayload(result, transformResult = DEFAULT_TRANSFORM_RESULT_FN) {
    switch (result.type) {
        case "RESPONSE":
            return toResponseResponsePayload(result, transformResult);
        case "MULTIPART_RESPONSE":
            return toMultiPartResponsePayload(result, transformResult);
        case "PUSH":
            return toPushResponsePayload(result, transformResult);
        default:
            throw new HttpError(500, "Cannot process result.");
    }
}
